@@ -68,6 +68,7 @@
 
 #include "base_wx/huginConfig.h"
 #include "hugin/AboutDialog.h"
+#include "hugin/RawImport.h"
 
 #if HUGIN_HSI
 #include "PluginItems.h"
@@ -140,34 +141,47 @@ bool PanoDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& file
 
     // try to add as images
     std::vector<std::string> filesv;
+    std::vector<std::string> rawFilesv;
     wxArrayString invalidFiles;
     for (unsigned int i=0; i< filenames.GetCount(); i++) {
         wxFileName file(filenames[i]);
-
-        if (file.GetExt().CmpNoCase(wxT("jpg")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("jpeg")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("tif")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("tiff")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("png")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("bmp")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("gif")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("pnm")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("sun")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("hdr")) == 0 ||
-            file.GetExt().CmpNoCase(wxT("viff")) == 0 )
+        // first check if raw file
+        // vigra::isImage reports also some raw files as image because
+        // technical some of them are tiff files
+        if (IsRawExtension(file.GetExt()))
         {
-            if(containsInvalidCharacters(filenames[i]))
+            if (containsInvalidCharacters(filenames[i]))
             {
                 invalidFiles.Add(file.GetFullPath());
             }
             else
             {
-                filesv.push_back((const char *)filenames[i].mb_str(HUGIN_CONV_FILENAME));
+                rawFilesv.push_back((const char *)file.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
             };
         }
+        else
+        {
+            // now check if drop file is a image
+            if (vigra::isImage(file.GetFullPath().mb_str(HUGIN_CONV_FILENAME)))
+            {
+                if (containsInvalidCharacters(filenames[i]))
+                {
+                    invalidFiles.Add(file.GetFullPath());
+                }
+                else
+                {
+                    filesv.push_back((const char *)filenames[i].mb_str(HUGIN_CONV_FILENAME));
+                };
+            };
+        };
+    }
+    // show warning about invalid file names
+    if (!invalidFiles.empty())
+    {
+        ShowFilenameWarning(mf, invalidFiles);
     }
     // we got some images to add.
-    if (filesv.size() > 0)
+    if (!filesv.empty())
     {
         // use a Command to ensure proper undo and updating of GUI parts
         wxBusyCursor();
@@ -183,13 +197,35 @@ bool PanoDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& file
             cmds.push_back(new PanoCommand::CenterPanoCmd(pano));
             PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::CombinedPanoCommand(pano, cmds));
         };
-
-    }
-    if(invalidFiles.size()>0)
+    };
+    if (!rawFilesv.empty())
     {
-        ShowFilenameWarning(mf, invalidFiles);
-    }
-
+        if (rawFilesv.size() == 1)
+        {
+            wxMessageDialog message(mf, _("You selected only one raw file. This is not recommended.\nAll raw files should be converted at once."),
+#ifdef _WIN32
+                _("Hugin"),
+#else
+                wxT(""),
+#endif
+                wxICON_EXCLAMATION | wxOK | wxCANCEL);
+            message.SetOKLabel(_("Convert anyway."));
+            if (message.ShowModal() != wxID_OK)
+            {
+                return true;
+            };
+        };
+        RawImportDialog dlg(mf, &pano, rawFilesv);
+        // check that raw files are from same camera and that all can be read
+        if (dlg.CheckRawFiles())
+        {
+            // now show dialog
+            if (dlg.ShowModal() == wxID_OK)
+            {
+                PanoCommand::GlobalCmdHist::getInstance().addCommand(dlg.GetPanoCommand());
+            };
+        };
+    };
     return true;
 }
 
@@ -373,6 +409,8 @@ MainFrame::MainFrame(wxWindow* parent, HuginBase::Panorama & pano)
     SetMenuBar(mainMenu);
     m_optOnlyActiveImages = (wxConfigBase::Get()->Read(wxT("/OptimizePanel/OnlyActiveImages"), 1l) != 0);
     m_optIgnoreLineCp = false;
+    // observe the panorama, this should be the first observer
+    pano.addObserver(this);
 
 #ifdef HUGIN_HSI
     wxMenuBar* menubar=GetMenuBar();
@@ -633,9 +671,6 @@ MainFrame::MainFrame(wxWindow* parent, HuginBase::Panorama & pano)
     wxYield();
     DEBUG_TRACE("");
 
-    // observe the panorama
-    pano.addObserver(this);
-
     // Set sizing characteristics
     //set minumum size
 #if defined __WXMAC__ || defined __WXMSW__
@@ -755,6 +790,15 @@ MainFrame::~MainFrame()
 
 void MainFrame::panoramaChanged(HuginBase::Panorama &pano)
 {
+    // set flag to indicate list should show correlation instead of error
+    if (m_showCorrelation == 2)
+    {
+        m_showCorrelation = 0;
+    };
+    if (m_showCorrelation == 1)
+    {
+        ++m_showCorrelation;
+    };
     wxToolBar* theToolBar = GetToolBar();
     wxMenuBar* theMenuBar = GetMenuBar();
     bool can_undo = PanoCommand::GlobalCmdHist::getInstance().canUndo();
@@ -906,6 +950,17 @@ void MainFrame::OnExit(wxCloseEvent & e)
     {
        preview_frame->Close(true);
     }
+    if(gl_preview_frame)
+    {
+        if(m_guiLevel==GUI_SIMPLE)
+        {
+            gl_preview_frame->StorePositionAndSize();
+        }
+        else
+        {
+            gl_preview_frame->Close(true);
+        };
+    }
 
     ImageCache::getInstance().flush();
     Destroy();
@@ -1200,12 +1255,7 @@ void MainFrame::OnAddImages( wxCommandEvent& event )
     if(cmd!=NULL)
     {
         PanoCommand::GlobalCmdHist::getInstance().addCommand(cmd);
-    }
-    else
-    {
-        // nothing to open
-        SetStatusText( _("Add Image: cancel"));
-    }
+    };
 
     DEBUG_TRACE("");
 }
@@ -1220,7 +1270,7 @@ void MainFrame::AddImages(wxArrayString& filenameArray)
             invalidFiles.Add(filenameArray[i]);
         };
     };
-    if(invalidFiles.size()>0)
+    if(!invalidFiles.empty())
     {
         ShowFilenameWarning(this, invalidFiles);
     }
@@ -1232,7 +1282,7 @@ void MainFrame::AddImages(wxArrayString& filenameArray)
         }
 
         // we got some images to add.
-        if (filesv.size() > 0) {
+        if (!filesv.empty()) {
             // use a Command to ensure proper undo and updating of GUI
             // parts
             wxBusyCursor();
@@ -1254,7 +1304,6 @@ void MainFrame::OnAddTimeImages( wxCommandEvent& event )
     };
 };
 
-
 void MainFrame::OnShowPanel(wxCommandEvent & e)
 {
     if(e.GetId()==XRCID("ID_SHOW_PANEL_MASK"))
@@ -1472,12 +1521,6 @@ void MainFrame::OnShowPrefs(wxCommandEvent & e)
     };
 }
 
-void MainFrame::UpdatePanels( wxCommandEvent& WXUNUSED(event) )
-{   // Maybe this can be invoced by the Panorama::Changed() or
-    // something like this. So no everytime update would be needed.
-    DEBUG_TRACE("");
-}
-
 void MainFrame::OnTogglePreviewFrame(wxCommandEvent & e)
 {
     DEBUG_TRACE("");
@@ -1838,6 +1881,7 @@ void MainFrame::OnFineTuneAll(wxCommandEvent & e)
         }
     }
     }
+    m_showCorrelation = 1;
     wxString result;
     result.Printf(_("%d points fine-tuned, %d points not updated due to low correlation\n\nHint: The errors of the fine-tuned points have been set to the correlation coefficient\nProblematic points can be spotted (just after fine-tune, before optimizing)\nby an error <= %.3f.\nThe error of points without a well defined peak (typically in regions with uniform color)\nwill be set to 0\n\nUse the Control Point list (F3) to see all points of the current project\n"),
                   nGood, nBad, corrThresh);
@@ -1853,7 +1897,7 @@ void MainFrame::OnRemoveCPinMasks(wxCommandEvent & e)
     if(pano.getCtrlPoints().size()<2)
         return;
     HuginBase::UIntSet cps=getCPinMasks(pano);
-    if(cps.size()>0)
+    if(!cps.empty())
     {
         PanoCommand::GlobalCmdHist::getInstance().addCommand(
                     new PanoCommand::RemoveCtrlPointsCmd(pano,cps)
@@ -2085,6 +2129,11 @@ wxString MainFrame::getProjectName()
     return m_filename;
 }
 
+bool MainFrame::IsShowingCorrelation() const
+{
+    return m_showCorrelation > 0;
+}
+
 void MainFrame::OnMRUFiles(wxCommandEvent &e)
 {
     size_t index = e.GetId() - wxID_FILE1;
@@ -2347,6 +2396,7 @@ void MainFrame::RunAssistant(wxWindow* mainWin, const wxString& userdefinedAssis
 
     // get assistant queue
     const wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+    wxArrayString tempfiles;
     HuginQueue::CommandQueue* commands;
     if (userdefinedAssistant.IsEmpty())
     {
@@ -2354,10 +2404,11 @@ void MainFrame::RunAssistant(wxWindow* mainWin, const wxString& userdefinedAssis
     }
     else
     {
-        commands = HuginQueue::GetAssistantCommandQueueUserDefined(pano, exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR), scriptFileName.GetFullPath(), userdefinedAssistant);
+        std::stringstream errors;
+        commands = HuginQueue::GetAssistantCommandQueueUserDefined(pano, exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR), scriptFileName.GetFullPath(), userdefinedAssistant, tempfiles, errors);
         if (commands->empty())
         {
-            wxMessageBox(_("The assistant queue is empty. This indicates an error in the user defined assistant file."),
+            wxMessageBox(_("The assistant queue is empty. This indicates an error in the user defined assistant file.") + "\n\n" + wxString(errors.str()),
                 _("Error"), wxOK | wxICON_ERROR, mainWin);
             //delete temporary files
             wxRemoveFile(scriptFileName.GetFullPath());
@@ -2375,6 +2426,20 @@ void MainFrame::RunAssistant(wxWindow* mainWin, const wxString& userdefinedAssis
 
     //delete temporary files
     wxRemoveFile(scriptFileName.GetFullPath());
+    if (!tempfiles.IsEmpty())
+    {
+        for (auto& f : tempfiles)
+        {
+            wxRemoveFile(f);
+        };
+    };
+    // check that GUI level is still approbiate
+    const GuiLevel reqGuiLevel = GetMinimumGuiLevel(pano);
+    if (reqGuiLevel > m_guiLevel)
+    {
+        // output of assistant requires higher GUI level
+        SetGuiLevel(reqGuiLevel);
+    };
     //if return value is non-zero, an error occurred in the assistant
     if(ret!=0)
     {
