@@ -24,12 +24,21 @@
  *
  */
 
+#ifdef _WIN32
+#include "wx/msw/wrapwin.h"
+#endif
 #include "Batch.h"
 #include <wx/stdpaths.h>
 #ifdef __WXMSW__
 #include <powrprof.h>
+#ifdef _MSC_VER
 #pragma comment(lib, "PowrProf.lib")
 #endif
+#endif
+#include <wx/dir.h>
+#ifndef __WXMSW__
+#include <sys/wait.h>
+#endif
 
 BEGIN_EVENT_TABLE(Batch, wxFrame)
     EVT_END_PROCESS(-1, Batch::OnProcessTerminate)
@@ -89,8 +98,8 @@ void Batch::AddProjectToBatch(wxString projectFile, wxString outputFile,Project:
 {
     wxFileName projectName(projectFile);
     wxFileName outName(outputFile);
-    projectName.Normalize();
-    outName.Normalize();
+    projectName.Normalize(wxPATH_NORM_ALL & ~wxPATH_NORM_ENV_VARS);
+    outName.Normalize(wxPATH_NORM_ALL & ~wxPATH_NORM_ENV_VARS);
 
     if(outputFile.Cmp(_T(""))!=0 || target==Project::DETECTING)
     {
@@ -174,13 +183,20 @@ void Batch::AppendBatchFile(wxString file)
             const bool skip = textStream.ReadLine().StartsWith(_T("T"));
 
             //we add project to internal list
-            if (line.IsEmpty())
+            if (id == -1)
             {
-                AddProjectToBatch(projectName,wxT(""),Project::DETECTING);
+                AddAppToBatch(projectName);
             }
             else
             {
-                AddProjectToBatch(projectName,line);
+                if (line.IsEmpty())
+                {
+                    AddProjectToBatch(projectName, wxT(""), Project::DETECTING);
+                }
+                else
+                {
+                    AddProjectToBatch(projectName, line);
+                };
             };
             //if status was RUNNING or PAUSED, we set it to FAILED
             if(status==(long)Project::RUNNING || status==(long)Project::PAUSED)
@@ -853,7 +869,7 @@ void Batch::RunNextInBatch()
 {
     bool value;
     bool repeat = true;
-    unsigned int i;
+    int i;
     while(((i=GetFirstAvailable())!=-1) && repeat)
     {
         //execute command line instructions
@@ -873,6 +889,11 @@ void Batch::RunNextInBatch()
         }
         else
         {
+            if (m_projList.Item(i).status == Project::MISSING)
+            {
+                // skip non-existing project files
+                continue;
+            };
             m_projList.Item(i).status=Project::RUNNING;
             m_running = true;
             if(m_projList.Item(i).target==Project::STITCHING)
