@@ -32,20 +32,33 @@
 #include "hugin_utils/alphanum.h"
 #include "hugin/config_defaults.h"
 #include "wx/mstream.h"
+#ifdef _WIN32
+// workaround for a conflict between exiv2 and wxWidgets/CMake built
+#define HAVE_PID_T 1
+#endif
 #include "exiv2/exiv2.hpp"
 #include "exiv2/preview.hpp"
 #ifdef _WIN32
-#include <CommCtrl.h>
+#include <commctrl.h>
 #endif
 #include "base_wx/LensTools.h"
 #include "panodata/StandardImageVariableGroups.h"
 
+enum
+{
+    ID_REMOVE_IMAGE = wxID_HIGHEST + 300,
+    ID_SPLIT_PANOS = wxID_HIGHEST + 301
+};
+
 BEGIN_EVENT_TABLE(FindPanoDialog,wxDialog)
     EVT_BUTTON(XRCID("find_pano_close"), FindPanoDialog::OnButtonClose)
     EVT_BUTTON(XRCID("find_pano_select_dir"), FindPanoDialog::OnButtonChoose)
     EVT_BUTTON(XRCID("find_pano_start_stop"), FindPanoDialog::OnButtonStart)
     EVT_BUTTON(XRCID("find_pano_add_queue"), FindPanoDialog::OnButtonSend)
     EVT_LISTBOX(XRCID("find_pano_list"), FindPanoDialog::OnSelectPossiblePano)
+    EVT_LIST_ITEM_RIGHT_CLICK(XRCID("find_pano_selected_thumbslist"), FindPanoDialog::OnListItemRightClick)
+    EVT_MENU(ID_REMOVE_IMAGE, FindPanoDialog::OnRemoveImage)
+    EVT_MENU(ID_SPLIT_PANOS, FindPanoDialog::OnSplitPanos)
     EVT_CLOSE(FindPanoDialog::OnClose)
 END_EVENT_TABLE()
 
@@ -78,9 +91,7 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     m_button_send=XRCCTRL(*this,"find_pano_add_queue",wxButton);
     m_button_close=XRCCTRL(*this,"find_pano_close",wxButton);
     m_textctrl_dir=XRCCTRL(*this,"find_pano_dir",wxTextCtrl);
-#if wxCHECK_VERSION(2,9,3)
     m_textctrl_dir->AutoCompleteDirectories();
-#endif
     m_cb_subdir=XRCCTRL(*this,"find_pano_subdir",wxCheckBox);
     m_statustext=XRCCTRL(*this,"find_pano_label",wxStaticText);
     m_list_pano=XRCCTRL(*this,"find_pano_list",wxCheckListBox);
@@ -157,12 +168,13 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     SelectListValue(m_ch_blender, i);
     m_button_send->Disable();
     m_thumbs = new wxImageList(THUMBSIZE, THUMBSIZE, true, 0);
-    wxListCtrl* thumbs_list = XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl);
-    thumbs_list->SetImageList(m_thumbs, wxIMAGE_LIST_NORMAL);
+    m_thumbsList = XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl);
+    m_thumbsList->SetImageList(m_thumbs, wxIMAGE_LIST_NORMAL);
+    m_thumbsList->Bind(wxEVT_MOTION, &FindPanoDialog::OnListMouseMove, this);
 #ifdef _WIN32
     // default image spacing is too big, wxWidgets does not provide direct 
     // access to the spacing, so using the direct API function
-    ListView_SetIconSpacing(thumbs_list->GetHandle(), THUMBSIZE + 20, THUMBSIZE + 20);
+    ListView_SetIconSpacing(m_thumbsList->GetHandle(), THUMBSIZE + 20, THUMBSIZE + 20);
 #endif
 };
 
@@ -199,13 +211,10 @@ FindPanoDialog::~FindPanoDialog()
 
 void FindPanoDialog::CleanUpPanolist()
 {
-    if(m_panos.size()>0)
+    while(!m_panos.empty())
     {
-        while(!m_panos.empty())
-        {
-            delete m_panos.back();
-            m_panos.pop_back();
-        };
+        delete m_panos.back();
+        m_panos.pop_back();
     };
 };
 
@@ -225,10 +234,10 @@ void FindPanoDialog::OnClose(wxCloseEvent& e)
 
 void FindPanoDialog::OnButtonClose(wxCommandEvent& e)
 {
-    if(m_panos.size()>0)
+    if(!m_panos.empty())
     {
         if(wxMessageBox(_("The list contains possibly unprocessed panoramas.\nIf you close the dialog, you will lose them.\nContinue anyway?"),
-                        _("Question"),wxYES_NO|wxICON_QUESTION,this)==wxNO)
+                        _("Question"),wxYES_NO|wxICON_WARNING,this)==wxNO)
         {
             return;
         };
@@ -260,10 +269,10 @@ void FindPanoDialog::OnButtonStart(wxCommandEvent& e)
         m_start_dir=m_textctrl_dir->GetValue();
         if(wxDir::Exists(m_start_dir))
         {
-            if(m_panos.size()>0)
+            if(!m_panos.empty())
             {
-                if(wxMessageBox(_("The list contains still not yet processed panoramas.\nIf you continue, they will be disregarded.\nDo you still want to continue?"),
-                                _("Question"),wxYES_NO|wxICON_QUESTION,this)==wxNO)
+                if (wxMessageBox(_("The list contains still not yet processed panoramas.\nIf you continue, they will be disregarded.\nDo you still want to continue?"),
+                                _("Question"),wxYES_NO|wxICON_WARNING,this)==wxNO)
                 {
                     return;
                 };
@@ -291,7 +300,7 @@ void FindPanoDialog::OnButtonStart(wxCommandEvent& e)
 
 void FindPanoDialog::OnButtonSend(wxCommandEvent& e)
 {
-    if(m_panos.size()==0)
+    if(m_panos.empty())
     {
         return;
     }
@@ -353,7 +362,7 @@ void FindPanoDialog::OnSelectPossiblePano(wxCommandEvent &e)
         XRCCTRL(*this, "find_pano_selected_lens", wxStaticText)->SetLabel(m_panos[selected]->GetLensName());
         XRCCTRL(*this, "find_pano_selected_focallength", wxStaticText)->SetLabel(m_panos[selected]->GetFocalLength());
         XRCCTRL(*this, "find_pano_selected_date_time", wxStaticText)->SetLabel(m_panos[selected]->GetStartString() + wxT(" (")+ m_panos[selected]->GetDuration() + wxT(")"));
-        m_panos[selected]->PopulateListCtrl(XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl), m_thumbs);
+        m_panos[selected]->PopulateListCtrl(m_thumbsList, m_thumbs, m_tooltips);
     }
     else
     {
@@ -361,8 +370,109 @@ void FindPanoDialog::OnSelectPossiblePano(wxCommandEvent &e)
         XRCCTRL(*this, "find_pano_selected_lens", wxStaticText)->SetLabel(wxEmptyString);
         XRCCTRL(*this, "find_pano_selected_focallength", wxStaticText)->SetLabel(wxEmptyString);
         XRCCTRL(*this, "find_pano_selected_date_time", wxStaticText)->SetLabel(wxEmptyString);
-        XRCCTRL(*this, "find_pano_selected_thumbslist", wxListCtrl)->DeleteAllItems();
+        m_thumbsList->DeleteAllItems();
         m_thumbs->RemoveAll();
+        m_tooltips.Clear();
+    };
+};
+
+void FindPanoDialog::OnListItemRightClick(wxListEvent &e)
+{
+    // build menu
+    wxMenu contextMenu;
+    const int selectedPano = m_list_pano->GetSelection();
+    if (m_panos[selectedPano]->GetImageCount() > 2)
+    {
+        contextMenu.Append(ID_REMOVE_IMAGE, _("Remove image from project"));
+    };
+    long imageIndex = -1;
+    imageIndex = m_thumbsList->GetNextItem(imageIndex, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+    if(imageIndex > 1 && imageIndex <= static_cast<long>(m_panos[selectedPano]->GetImageCount()) - 2)
+    {
+        contextMenu.Append(ID_SPLIT_PANOS, _("Split here into two panoramas"));
+    }
+    // show popup menu
+    if (contextMenu.GetMenuItemCount() > 0)
+    {
+        PopupMenu(&contextMenu);
+    };
+};
+
+void FindPanoDialog::OnRemoveImage(wxCommandEvent &e)
+{
+    const int selectedPano = m_list_pano->GetSelection();
+    if (selectedPano != wxNOT_FOUND)
+    {
+        if (m_panos[selectedPano]->GetImageCount() > 2)
+        {
+            long imageIndex = -1;
+            imageIndex = m_thumbsList->GetNextItem(imageIndex, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+            if (imageIndex != wxNOT_FOUND)
+            {
+                // remove image from possible pano
+                m_panos[selectedPano]->RemoveImage(imageIndex);
+                // now remove from the wxListCtrl
+                m_thumbsList->DeleteItem(imageIndex);
+                // update the pano list
+                m_list_pano->SetString(selectedPano, m_panos[selectedPano]->GetItemString(m_start_dir));
+                // update the labels above
+                wxCommandEvent dummy;
+                OnSelectPossiblePano(dummy);
+            };
+        };
+    };
+};
+
+void FindPanoDialog::OnSplitPanos(wxCommandEvent &e)
+{
+    const int selectedPano = m_list_pano->GetSelection();
+    if (selectedPano != wxNOT_FOUND)
+    {
+        long imageIndex = -1;
+        imageIndex = m_thumbsList->GetNextItem(imageIndex, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+        if (imageIndex != wxNOT_FOUND)
+        {
+            if (imageIndex > 1 && imageIndex <= static_cast<long>(m_panos[selectedPano]->GetImageCount()) - 2)
+            {
+                // do split
+                PossiblePano* newSubPano = m_panos[selectedPano]->SplitPano(imageIndex);
+                if (newSubPano->GetImageCount() > 0)
+                {
+                    // insert new pano into internal list
+                    m_panos.insert(m_panos.begin() + selectedPano + 1, newSubPano);
+                    // update pano list
+                    m_list_pano->SetString(selectedPano, m_panos[selectedPano]->GetItemString(m_start_dir));
+                    int newItem = m_list_pano->Insert(m_panos[selectedPano + 1]->GetItemString(m_start_dir), selectedPano + 1);
+                    m_list_pano->Check(newItem, true);
+                    // update display
+                    wxCommandEvent dummy;
+                    OnSelectPossiblePano(dummy);
+                }
+                else
+                {
+                    wxBell();
+                    delete newSubPano;
+                };
+            };
+        };
+    };
+}
+
+void FindPanoDialog::OnListMouseMove(wxMouseEvent & e)
+{
+    if (m_tooltips.IsEmpty())
+    {
+        return;
+    }
+    int flags;
+    long item = m_thumbsList->HitTest(e.GetPosition(), flags);
+    if (item != wxNOT_FOUND && (flags&wxLIST_HITTEST_ONITEM) && item < m_tooltips.GetCount())
+    {
+        m_thumbsList->SetToolTip(m_tooltips[item]);
+    }
+    else
+    {
+        m_thumbsList->UnsetToolTip();
     };
 };
 
@@ -447,7 +557,7 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
         //allow processing events
         wxGetApp().Yield(true);
     };
-    if(!m_stopped && newPanos.size()>0)
+    if(!m_stopped && !newPanos.empty())
     {
         for(size_t i=0; i<newPanos.size(); i++)
         {
@@ -482,8 +592,8 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
         m_button_start->SetLabel(_("Start"));
         EnableButtons(true);
         //enable send button if at least one panorama found
-        m_button_send->Enable(m_panos.size()>0);
-        if(m_panos.size()>0)
+        m_button_send->Enable(!m_panos.empty());
+        if(!m_panos.empty())
         {
             m_statustext->SetLabel(wxString::Format(_("Found %d possible panoramas."), static_cast<int>(m_panos.size())));
         }
@@ -497,7 +607,7 @@ void FindPanoDialog::SearchInDir(wxString dirstring, const bool includeSubdir, c
 
 PossiblePano::~PossiblePano()
 {
-    if(m_images.size()>0)
+    if(!m_images.empty())
     {
         for(ImageSet::reverse_iterator it=m_images.rbegin(); it!=m_images.rend(); ++it)
         {
@@ -688,9 +798,21 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
         return wxEmptyString;
     };
     //generate panorama
+    double redBalanceAnchor = (*m_images.begin())->getExifRedBalance();
+    double blueBalanceAnchor = (*m_images.begin())->getExifBlueBalance();
+    if (fabs(redBalanceAnchor)<1e-2)
+    {
+        redBalanceAnchor = 1;
+    };
+    if (fabs(blueBalanceAnchor)<1e-2)
+    {
+        blueBalanceAnchor = 1;
+    };
     HuginBase::Panorama pano;
     for(ImageSet::iterator it=m_images.begin(); it!=m_images.end(); ++it)
     {
+        (*it)->setWhiteBalanceRed((*it)->getExifRedBalance() / redBalanceAnchor);
+        (*it)->setWhiteBalanceBlue((*it)->getExifBlueBalance() / blueBalanceAnchor);
         pano.addImage(*(*it));
     };
     //assign all images the same lens number
@@ -698,28 +820,9 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
     HuginBase::ImageVariableGroup& lenses = variable_groups.getLenses();
     if(pano.getNrOfImages()>1)
     {
-        double redBalanceAnchor=pano.getImage(pano.getOptions().colorReferenceImage).getExifRedBalance();
-        double blueBalanceAnchor=pano.getImage(pano.getOptions().colorReferenceImage).getExifBlueBalance();
-        if(fabs(redBalanceAnchor)<1e-2)
-        {
-            redBalanceAnchor=1;
-        };
-        if(fabs(blueBalanceAnchor)<1e-2)
-        {
-            blueBalanceAnchor=1;
-        };
         for(unsigned int i=1; i<pano.getNrOfImages(); i++)
         {
-            HuginBase::SrcPanoImage img = pano.getSrcImage(i);
-            double ev=img.getExposureValue();
-            lenses.switchParts(i,lenses.getPartNumber(0));
-            lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_ExposureValue, i);
-            img.setExposureValue(ev);
-            lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceRed, i);
-            lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceBlue, i);
-            img.setWhiteBalanceRed(img.getExifRedBalance()/redBalanceAnchor);
-            img.setWhiteBalanceBlue(img.getExifBlueBalance()/blueBalanceAnchor);
-            pano.setSrcImage(i, img);
+            lenses.switchParts(i, lenses.getPartNumber(0));
         };
     };
     if (pano.hasPossibleStacks())
@@ -776,6 +879,7 @@ wxString PossiblePano::GeneratePanorama(NamingConvention nc, bool createLinks, H
     opts.tiff_saveROI = config->Read(wxT("Nona/CroppedImages"),HUGIN_NONA_CROPPEDIMAGES)!=0;
     opts.hdrMergeMode = HuginBase::PanoramaOptions::HDRMERGE_AVERAGE;
     opts.hdrmergeOptions = HUGIN_HDRMERGE_ARGS;
+    opts.verdandiOptions = config->Read(wxT("/VerdandiDefaultArgs"), wxEmptyString).mb_str(wxConvLocal);
     pano.setOptions(opts);
     // set optimizer switches
     pano.setOptimizerSwitch(HuginBase::OPT_POSITION);
@@ -814,6 +918,10 @@ wxString PossiblePano::GetLensName()
 
 wxString PossiblePano::GetFocalLength()
 {
+    if (!m_images.empty())
+    {
+        return FormatString::GetFocalLength(*m_images.begin());
+    };
     return wxString::Format(wxT("%0.1f mm"), m_focallength);
 };
 
@@ -835,10 +943,11 @@ wxString PossiblePano::GetDuration()
     };
 };
 
-void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
+void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs, wxArrayString& tooltips)
 {
     list->DeleteAllItems();
     thumbs->RemoveAll();
+    tooltips.Clear();
     wxBusyCursor cursor;
     for (ImageSet::iterator it = m_images.begin(); it != m_images.end(); ++it)
     {
@@ -891,6 +1000,27 @@ void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
                     x = (THUMBSIZE - newWidth) / 2;
                     y = 0;
                 }
+                // rotate according to orientation tag
+                if ((*it)->getRoll() == 90)
+                {
+                    rawImage = rawImage.Rotate90();
+                    std::swap(x, y);
+                }
+                else
+                {
+                    if ((*it)->getRoll() == 270)
+                    {
+                        rawImage = rawImage.Rotate90(false);
+                        std::swap(x, y);
+                    }
+                    else
+                    {
+                        if ((*it)->getRoll() == 180)
+                        {
+                            rawImage = rawImage.Rotate180();
+                        };
+                    };
+                };
                 // create final bitmap with centered thumbnail
                 wxBitmap bitmap(THUMBSIZE, THUMBSIZE);
                 wxMemoryDC dc(bitmap);
@@ -909,5 +1039,66 @@ void PossiblePano::PopulateListCtrl(wxListCtrl* list, wxImageList* thumbs)
         // create item in thumb list
         wxFileName fn(wxString((*it)->getFilename().c_str(), HUGIN_CONV_FILENAME));
         list->InsertItem(list->GetItemCount(), fn.GetFullName(), index);
+        // build tooltip text
+        tooltips.Add(fn.GetFullName() + "\n" + FormatString::GetAperture(*it) + ", "
+            + FormatString::GetExposureTime(*it) + ", " + _("ISO") + FormatString::GetIso(*it)
+            + "\n" + FormatString::GetExifDateTime(*it));
     };
 };
+
+void PossiblePano::RemoveImage(const unsigned int index)
+{
+    // remove image with given index
+    if (index < m_images.size())
+    {
+        ImageSet::iterator item = m_images.begin();
+        std::advance(item, index);
+        delete *item;
+        m_images.erase(item);
+        //update the internal times
+        UpdateDateTimes();
+    };
+}
+
+PossiblePano* PossiblePano::SplitPano(const unsigned int index)
+{
+    PossiblePano* newPano = new PossiblePano();
+    if (index < m_images.size())
+    {
+        // now move all images to right pano
+        ImageSet allImages = m_images;
+        m_images.clear();
+        ImageSet::iterator img = allImages.begin();
+        while (m_images.size() < index && img != allImages.end())
+        {
+            m_images.insert(*img);
+            ++img;
+        };
+        while (img != allImages.end())
+        {
+            newPano->AddSrcPanoImage(*img);
+            ++img;
+        }
+        UpdateDateTimes();
+    };
+    return newPano;
+}
+
+void PossiblePano::UpdateDateTimes()
+{
+    // update internal stored start and end time
+    m_dt_start = GetDateTime(*m_images.begin());
+    m_dt_end = m_dt_start;
+    for (auto& img : m_images)
+    {
+        wxDateTime dt = GetDateTime(img);
+        if (dt.IsEarlierThan(m_dt_start))
+        {
+            m_dt_start = dt;
+        }
+        if (dt.IsLaterThan(m_dt_end))
+        {
+            m_dt_end = dt;
+        };
+    };
+}
