@@ -123,23 +123,6 @@ bool SrcPanoImage::horizontalWarpNeeded()
     return false;
 }
 
-void BaseSrcPanoImage::setDefaults()
-{
-    /* Some of the vectors are difficult to initalise with the variables list
-     * header, so we make some local variables which are used in it.
-     */
-    // Radial Distortion defaults
-    std::vector<double> distortion_default(4, 0.0);
-    distortion_default[3] = 1;
-    
-    std::vector<double> RadialVigCorrCoeff_default(4, 0.0);
-    RadialVigCorrCoeff_default[0] = 1;
-    HuginBase::MaskPolygonVector defaultMaskVector;
-#define image_variable( name, type, default_value ) m_##name.setData(default_value);
-#include "image_variables.h"
-#undef image_variable
-}
-
 bool SrcPanoImage::isInside(vigra::Point2D p, bool ignoreMasks) const
 {
     bool insideCrop=false;
@@ -235,7 +218,7 @@ bool BaseSrcPanoImage::operator==(const BaseSrcPanoImage & other) const
 double SrcPanoImage::getVar(const std::string & code) const
 {
     DEBUG_TRACE("");
-    assert(code.size() > 0);
+    assert(!code.empty());
 #define image_variable( name, type, default_value ) \
     if (PTOVariableConverterFor##name::checkApplicability(code)) \
         return PTOVariableConverterFor##name::getValueFromVariable(code, m_##name );\
@@ -251,7 +234,7 @@ double SrcPanoImage::getVar(const std::string & code) const
 void SrcPanoImage::setVar(const std::string & code, double val)
 {
     DEBUG_TRACE("Var:" << code << " value: " << val);
-    assert(code.size() > 0);
+    assert(!code.empty());
 #define image_variable( name, type, default_value ) \
     if (PTOVariableConverterFor##name::checkApplicability(code)) \
         {PTOVariableConverterFor##name::setValueFromVariable(code, m_##name, val);}\
@@ -342,12 +325,10 @@ bool SrcPanoImage::readEXIF()
     Exiv2::Image::AutoPtr image;
     try {
         image = Exiv2::ImageFactory::open(filename.c_str());
-    }catch(...) {
-        std::cerr << __FILE__ << " " << __LINE__ << " Error opening file" << std::endl;
-        return false;
     }
-    if (image.get() == 0) {
-        std::cerr << "Unable to open file to read EXIF data: " << filename << std::endl;
+    catch (const Exiv2::Error& e)
+    {
+        std::cerr << "Exiv2: Error reading metadata (" << e.what() << ")" << std::endl;
         return false;
     }
 
@@ -539,7 +520,7 @@ bool SrcPanoImage::readEXIF()
         };
     };
     // check results, if 35 mm focal length is too small reset crop factor to 0
-    if (focalLength > 0 && cropFactor > 0 && focalLength*cropFactor < 8)
+    if (focalLength > 0 && cropFactor > 0 && focalLength*cropFactor < 6)
     {
         cropFactor = 0;
         // check alternative way to calculate crop factor, e.g. when focal length and focal length in 35 mm are given
@@ -547,7 +528,7 @@ bool SrcPanoImage::readEXIF()
         const double newCropFactor = Exiv2Helper::getCropFactor(exifData, getWidth(), getHeight());
         if (newCropFactor > 0)
         {
-            if (focalLength*newCropFactor > 8)
+            if (focalLength*newCropFactor >= 6)
             {
                 cropFactor = newCropFactor;
             }
@@ -676,9 +657,27 @@ std::string SrcPanoImage::getDBLensName() const
     return std::string();
 };
 
+bool isFisheye(const BaseSrcPanoImage::Projection& proj)
+{
+    switch (proj)
+    {
+        case BaseSrcPanoImage::CIRCULAR_FISHEYE:
+        case BaseSrcPanoImage::FULL_FRAME_FISHEYE:
+        case BaseSrcPanoImage::FISHEYE_ORTHOGRAPHIC:
+        case BaseSrcPanoImage::FISHEYE_STEREOGRAPHIC:
+        case BaseSrcPanoImage::FISHEYE_EQUISOLID:
+        case BaseSrcPanoImage::FISHEYE_THOBY:
+            return true;
+        default:
+            return false;
+    };
+    return false;
+};
+
 bool SrcPanoImage::readProjectionFromDB()
 {
     bool success=false;
+    double oldFocal = 0;
     const std::string lensname = getDBLensName();
     const double focal = getExifFocalLength();
     if (!lensname.empty())
@@ -687,22 +686,37 @@ bool SrcPanoImage::readProjectionFromDB()
         Projection dbProjection;
         if(lensDB.GetProjection(lensname, dbProjection))
         {
+            oldFocal = calcFocalLength(getProjection(), getHFOV(), getCropFactor(), getSize());
             setProjection(dbProjection);
-            success=true;
+            success = true;
         };
         if (focal>0)
         {
             double fov;
-            if (lensDB.GetFov(lensname, focal, fov))
+            // read fov only for non rectilinear images
+            // for these relay on the EXIF data, because often user manage to store
+            // wrong values in the database, so ignore them for rectilinear images
+            if (getProjection() != RECTILINEAR && lensDB.GetFov(lensname, focal, fov))
             {
-                // ignore values if fov is bigger than 120 deg
-                // then probably an invalid value was written earlier
-                if (getProjection() == RECTILINEAR && fov < 120)
-                {
-                    // calculate FOV for given image, take different aspect ratios into account
-                    const double newFocal = calcFocalLength(getProjection(), fov, getCropFactor(), vigra::Size2D(3000, 2000));
-                    const double newFov = calcHFOV(getProjection(), newFocal, getCropFactor(), getSize());
-                    setHFOV(newFov);
+                // calculate FOV for given image, take different aspect ratios into account
+                const double newFocal = calcFocalLength(getProjection(), fov, getCropFactor(), vigra::Size2D(3000, 2000));
+                const double newFov = calcHFOV(getProjection(), newFocal, getCropFactor(), getSize());
+                setHFOV(newFov);
+                oldFocal = 0;
+                // for fisheye lenses read also automatically the distortions parameters from lens db
+                // because fisheye often don't follow exactly one of the projection models and need
+                // the distortion parameters to model the real projection of the used fisheye lens
+                if(isFisheye(getProjection()))
+                { 
+                    std::vector<double> dist;
+                    if (lensDB.GetDistortion(lensname, focal, dist))
+                    {
+                        if (dist.size() == 3)
+                        {
+                            dist.push_back(1.0 - dist[0] - dist[1] - dist[2]);
+                            setRadialDistortion(dist);
+                        };
+                    };
                 };
             };
             vigra::Rect2D dbCropRect;
@@ -712,6 +726,12 @@ bool SrcPanoImage::readProjectionFromDB()
                 setCropRect(dbCropRect);
             };
         };
+        // updated fov after changing projection, if not already done with value from database
+        if (success && oldFocal > 0)
+        {
+            const double newFov = calcHFOV(getProjection(), oldFocal, getCropFactor(), getSize());
+            setHFOV(newFov);
+        };
     };
     return success;
 };
@@ -950,13 +970,13 @@ void SrcPanoImage::clearActiveMasks()
 
 bool SrcPanoImage::hasMasks() const
 {
-    return m_Masks.getData().size()>0;
+    return !m_Masks.getData().empty();
 };
 
 bool SrcPanoImage::hasPositiveMasks() const
 {
     MaskPolygonVector masks=m_Masks.getData();
-    if(masks.size()>0)
+    if(!masks.empty())
     {
         for(unsigned int i=0;i<masks.size();i++)
         {
@@ -971,12 +991,12 @@ bool SrcPanoImage::hasPositiveMasks() const
 
 bool SrcPanoImage::hasActiveMasks() const
 {
-    return m_ActiveMasks.getData().size()>0;
+    return !m_ActiveMasks.getData().empty();
 };
  
 void SrcPanoImage::printMaskLines(std::ostream &o, unsigned int newImgNr) const
 {
-    if(m_Masks.getData().size()>0)
+    if(!m_Masks.getData().empty())
         for(unsigned int i=0;i<m_Masks.getData().size();i++)
             m_Masks.getData()[i].printPolygonLine(o, newImgNr);
 };
