@@ -21,10 +21,6 @@ find . -name '*.xrc' | sort | while read line; do wxrc -g $line >> ${WDIR}/xrc.c
 cd ${WDIR}
 echo "Done preparing rc files"
        
-echo "Filtering out ignored strings"
-./filter-ignored-strings.py xrc.cpp
-echo "Done filtering"
-       
 echo "Extracting messages"
 cd ${BASEDIR}
 # we use simple sorting to make sure the lines do not jump around too much from system to system
